import json
import logging
import smtplib
import sys
import time
import traceback
import datetime
import os
import random
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formataddr
import re
import pandas as pd
import requests
from lxml import etree

logger = logging.getLogger(__name__)


class SoYoung(object):
    def __init__(self):
        self.url = "https://api.soyoung.com/v8/superList/index"
        self.headers = {'_time': '1626769752',
                        'ab_id': 'C521C79519A5D544390E60FEA08B32DB',
                        "app_id": 42,
                        "area_belong": 4,
                        "channel": 1,
                        # "cityId": str(city_id),
                        "device_id": 196374256,
                        "device_model": "iPhone12,1",
                        'device_version': '13.6.1',
                        "event": "--Boundary+D46DCD61FE6FA268",
                        "filter": {},
                        "from_site": 1,
                        "gps_city_id": 1,
                        "idfa": "057F28DF-20B8-488F-A285-931367FCC110",
                        "is_tf": 0,
                        "item_id": "--Boundary+D46DCD61FE6FA268",
                        # "keyword": str(keyword),
                        "list_name": "sy_app_superlist_search_page",
                        "lver": "8.28.2",
                        # "page": page,
                        "page_size": 20,
                        "push_app_id": 42,
                        "request_id": "14d1e2b53ca644242ec7ccd7316a0aa2",
                        "s_mei_device_id": "20200317131719d8bcbc37c54be511421dc3ebf7f1d0a801036b566bd47092",
                        "s_meng_device_id": "D2VCzq4o472Ur7QtdVY6RlcjO6h3455JlJ+OC39JcQC7sX6a",
                        "schemecard": "--Boundary+D46DCD61FE6FA268",
                        "sys": 1,
                        "tab": "mix",
                        "uid": "48804194",
                        "vistor_uid": "",
                        "xy_device_token": "33fa06111dea535c88cc07521f2e466c91",
                        "xy_token": "ad970db3d79f0833d1d25d3942068585"
                        }

    def get_keynote_sentence(self, content):
        try:
            ss = content.encode('utf-16', 'surrogatepass').decode('utf-16')
            dr = re.compile(r"<[^>]+>", re.S)
            str_re = dr.sub("", ss)
            para = re.sub('([；。！？\?])([^”’])', r"\1\n\2", str_re)  # 单字符断句符
            para = re.sub('(\.{6})([^”’])', r"\1\n\2", para)  # 英文省略号
            para = re.sub('(\…{2})([^”’])', r"\1\n\2", para)  # 中文省略号
            para = re.sub('([；。！？\?][”’])([^，。！？\?])', r'\1\n\2', para)
            para = para.rstrip()  # 段尾如果有多余的\n就去掉它
            return para
        except:
            logging.error("catch exception,logins:%s" % traceback.format_exc())
            return []

    def search(self):
        try:
            # 每次请求之前先暂停几秒 防止被ban
            s = random.random()
            time.sleep(s)
            response_res = requests.post(self.url, self.headers, verify=False)
            if response_res.status_code == 200 and response_res.text:
                response = json.loads(response_res.text)
                return response
            elif response_res.status_code == 403:
                return self.search()
            else:
                print("发生错误 停止请求")
        except:
            return print("发生错误 停止请求")

    def response_analysis(self, response=None, city_id=-1, city_name=None, keyword=None, all_skuids=[],
                          get_data_file=None, other_city_count=0, break_flat=False):
        """
        解析获取到的数据
        """
        try:
            responseData = response.get("responseData", {}).get("data")
            for item in responseData:
                if item.get("type") == "feed_area":
                    if item.get("items", {}).get("feed_list", []):
                        for data in item.get("items", {}).get("feed_list", []):
                            if data.get("type") == "feed_shop_diallel":
                                for service in data.get("items", []):
                                    service_data = service.get("data")
                                    district_2 = service_data.get("district_2")
                                    if str(district_2) == str(city_id):
                                        service_info = dict()
                                        service_info['skuid'] = service_data.get("pid")
                                        service_info['美购id'] = service_data.get("spu_id")
                                        # service_info['医生名'] = service_data.get("doctor_name")
                                        service_info['医院名称'] = service_data.get("hospital_name")
                                        service_info['sku原价'] = service_data.get("price_origin")
                                        service_info['sku活动价'] = service_data.get("price_online")
                                        service_info['机构等级'] = service_data.get("avg_score")
                                        service_info['美购名称'] = self.get_keynote_sentence(service_data.get("title"))
                                        service_info['销量'] = service_data.get("order_cnt")
                                        icon_data = service_data.get("icons", [])
                                        service_info['可用尾款券'] = service_data.get("wei_kuan_list", [])
                                        service_info['可领取预约金优惠券'] = [
                                            service_data.get("new_user_text", "")] if service_data.get(
                                            "new_user_text", "") else []
                                        for item in icon_data:
                                            if "预约金满" in item:
                                                service_info['可领取预约金优惠券'].append(item)
                                            elif "尾款满" in item:
                                                service_info['可用尾款券'].append(item)

                                        service_info['query词'] = keyword
                                        service_info['城市'] = city_name
                                        service_info['平台'] = "新氧"
                                        service_info['链接'] = "https://m.soyoung.com/normal/cpwap{}".format(
                                            service_info['skuid'])

                                        if service_data.get("pid") not in all_skuids:
                                            get_data_file.write(json.dumps(service_info))
                                            get_data_file.write("\n")
                                            print("write success")
                                    else:
                                        other_city_count += 1
                    else:
                        print("break")
                        break_flat = True
                        break
        except:
            pass

    def get_service_info(self, city_id=-1, keyword="", city_name="", all_skuids=[], get_data_file=None):
        break_flat = False
        other_city_count = 0
        for page in range(1, 500):
            # other_city_count <10 代表已经结束
            if break_flat == False and other_city_count < 10:
                self.headers['cityId'] = str(city_id)
                self.headers['keyword'] = str(keyword)
                self.headers['page'] = str(page)
                response = self.search()
                if response:
                    self.response_analysis(response=response, city_id=city_id, keyword=keyword, city_name=city_name,
                                           all_skuids=all_skuids, get_data_file=get_data_file,
                                           other_city_count=other_city_count, break_flat=break_flat)
                else:
                    print(city_id, keyword, "爬取失败")
            else:
                print(page, city_id, keyword, "本地已爬完")


def main(city_tag=""):
    begin = time.time()
    nowday = datetime.datetime.now()
    today = str(nowday).split()[0]
    file_name = "save_data_" + str(today) + city_tag + ".txt"

    get_data_file = open(file_name, "a+", encoding="utf-8")

    cityIdMapping = {'北京': '1', '上海': '9', '重庆': '22', '广州市': '289', '深圳市': '291', '郑州市': '240', '武汉市': '258',
                     '长沙市': '275', '南京市': '162', '成都市': '385', '西安市': '438', '杭州市': '175'}

    keywords = ['瘦脸针', '双眼皮', '光子嫩肤', '吸脂', '水光针', '玻尿酸', '热玛吉',
                '脱毛', '瘦腿针', '鼻综合', '瘦肩针', '下颌角', '线雕', '超声刀',
                '美白针', '眼综合', '隆鼻', '菲洛嘉', '下巴', '热拉提', '点阵激光',
                '面部吸脂', '开内眼角', '嗨体', '牙齿矫正', '皮秒', '超皮秒', '植发',
                '黄金微针', '隆胸', '微针', '光子嫩肤', '祛斑', '小气泡', '嗨体熊猫针',
                '熊猫针', '果酸焕肤', '自体脂肪隆胸', '7D聚拉提', '瘦脸', '埋线双眼皮',
                '菲洛嘉水光针', '双眼皮修复', '欧洲之星', '脂肪填充', '溶脂针', '法令纹',
                '鼻基底', '全切双眼皮', '颧骨内推', '鼻子', '光子嫩肤m22', '下颌缘提升',
                'm22', '鼻翼缩小', '欧洲之星fotona4d', '自体脂肪全面部填充', '玻尿酸丰唇',
                '水光', '嗨体祛颈纹', '假体隆胸', '英诺小棕瓶', '黄金微雕', '眼袋', '乔雅登',
                '除皱', '颧骨', '艾莉薇', '抽脂', '瘦腿', '玻尿酸丰下巴', '纹眉', '伊肤泉微针',
                '美白', '厚唇改薄', '面部线雕', '祛疤', '伊婉V', '超皮秒祛斑', '除皱针', '开眼角',
                '海菲秀', '假体下巴', '刷酸', '泪沟', '拉皮', '全身吸脂', '缩鼻翼']

    soyoung = SoYoung()

    city_list = [city_tag]
    all_skuids = []
    for city_name in city_list:
        city_id = cityIdMapping.get(city_name)
        for word in keywords:
            soyoung.get_service_info(city_id=city_id, keyword=word, city_name=city_name,
                                     all_skuids=all_skuids, get_data_file=get_data_file)

    get_data_file.close()
    print(time.time() - begin)
    print("全部爬取完成")


if __name__ == '__main__':
    args = sys.argv[1]
    main(city_tag=args)
